/**
 * TEXAS INSTRUMENTS TEXT FILE LICENSE
 * 
 * 
 * Copyright (c) 2014 - 2017 Texas Instruments Incorporated
 * 
 * All rights reserved not granted herein.
 * 
 * Limited License.  
 * 
 * Texas Instruments Incorporated grants a world-wide, royalty-free, non-exclusive license under copyrights and patents it now or hereafter owns or controls to make, have made, use, import, offer to sell and sell ("Utilize") this software subject to the terms herein.  With respect to the foregoing patent license, such license is granted  solely to the extent that any such patent is necessary to Utilize the software alone.  The patent license shall not apply to any combinations which include this software, other than combinations with devices manufactured by or for TI ("TI Devices").  No hardware patent is licensed hereunder.
 * 
 * Redistributions must preserve existing copyright notices and reproduce this license (including the above copyright notice and the disclaimer and (if applicable) source code license limitations below) in the documentation and/or other materials provided with the distribution
 * 
 * Redistribution and use in binary form, without modification, are permitted provided that the following conditions are met:
 * 
 * *	No reverse engineering, decompilation, or disassembly of this software is permitted with respect to any software provided in binary form.
 * 
 * *	any redistribution and use are licensed by TI for use only with TI Devices.
 * 
 * *	Nothing shall obligate TI to provide you with source code for the software licensed and provided to you in object code.
 * 
 * If software source code is provided to you, modification and redistribution of the source code are permitted provided that the following conditions are met:
 * 
 * *	any redistribution and use of the source code, including any resulting derivative works, are licensed by TI for use only with TI Devices.
 * 
 * *	any redistribution and use of any object code compiled from the source code and any resulting derivative works, are licensed by TI for use only with TI Devices.
 * 
 * Neither the name of Texas Instruments Incorporated nor the names of its suppliers may be used to endorse or promote products derived from this software without specific prior written permission.
 * 
 * DISCLAIMER.
 * 
 * THIS SOFTWARE IS PROVIDED BY TI AND TI'S LICENSORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TI'S LICENSORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * \file BSL432_Command_Interpreter.c
 * \brief The command interpreter is a super loop that calls corresponding BSL
 *        functions based on received commands.
 */
 
 /*
 * Changelog
 * 2017-11-28  0007  FS  Add different header file for MSP432P4111, MSP432P411V, and MSP432P411Y.
 */

#include "BSL432_Command_Interpreter.h"
#include "BSL432_Command_Definitions.h"
#include "BSL432_API.h"
#include "BSL432_Peripheral_Interface.h"
#if defined (__MSP432P401R__) || defined (__MSP432P401M__)
   #include "MSP432P401x/BSL432_device_file.h"
#elif defined (__MSP432P4111__) || defined (__MSP432P411V__) || defined (__MSP432P411Y__)
   #include "MSP432P4111/BSL432_device_file.h"
#else
#endif

#ifdef _DEBUG
    #warning DEBUG BUILD!
    #pragma RETAIN(BSL432_VersionVendor)
    #pragma DATA_SECTION(BSL432_VersionVendor, ".BSL432_VERSION_VENDOR")
    const uint8_t BSL432_VersionVendor[2] = {0xFF, 0xFF};  // Debug build uses vendor id 0xFFFF
#else
    #pragma RETAIN(BSL432_VersionVendor)
    #pragma DATA_SECTION(BSL432_VersionVendor, ".BSL432_VERSION_VENDOR")
    const uint8_t BSL432_VersionVendor[2] = {0x00, 0x00};  // TI production build uses vendor id 0x0000
#endif

#pragma RETAIN(BSL432_VersionCommandInterpreter)
#pragma DATA_SECTION(BSL432_VersionCommandInterpreter, ".BSL432_VERSION_CI")
const uint8_t BSL432_VersionCommandInterpreter[2] = {0x00, 0x03};

#pragma RETAIN(BSL432_VersionID)
#pragma DATA_SECTION(BSL432_VersionID, ".BSL432_VERSION_ID")
const uint8_t BSL432_VersionID[2] = BSL432_VERSION;

uint8_t *BSL432_ReceiveBuffer = 0;
uint8_t *BSL432_SendBuffer = 0;
uint32_t BSL432_ReceiveBufferSize = 0;
uint32_t BSL432_SendBufferSize = 0;

uint32_t BSL432_BSL_PARAMS = 0;  // Store BSL parameters given by boot code or user application

/**
 * BSL main function.
 * Handles initialization and runs the command interpreter as super loop.
 *
 * \param bslParams BSL hardware invoke parameters
 *
 * \return None
 */
void main(uint32_t bslParams)
{
    BSL432_eventFlags BSL432_eventFlag = Flags_rxPacketOngoing;

#ifdef _DEBUG
    /* Indicate BSL invocation on P1.0 */
    MAP_GPIO_setAsOutputPin(GPIO_PORT_P1, GPIO_PIN0);
    MAP_GPIO_setOutputHighOnPin(GPIO_PORT_P1, GPIO_PIN0);
#endif

    BSL432_API_init();

#ifdef _DEBUG
    /* Set BSL parameters for debug */
    BSL432_BSL_PARAMS = 0x7C48EF80U;
#else
    /* Retrieve BSL parameters from RAM and store them globally */
    BSL432_BSL_PARAMS = *((uint32_t *)(BSL432_RAM_BSL_PARAMS));
#endif

    BSL432_PI_init();

    BSL432_API_initTimeout();

    BSL432_PI_detectInterface();

    // BSL super loop
    // Exit BSL via reboot reset command
    while(1)
    {
        BSL432_eventFlag = BSL432_PI_receivePacket();
        if(BSL432_eventFlag == Flags_data_received)
        {
            BSL432_CI_interpretCommand();
        }
    }

}

/**
 * BSL Command Interpreter
 * Interprets the command contained in the data buffer and
 * calls the appropriate BSL API function
 *
 * \return None
 */
void BSL432_CI_interpretCommand()
{
    uint8_t command = BSL432_ReceiveBuffer[3];
    uint32_t addr = BSL432_ReceiveBuffer[4];
    addr |= ((uint32_t)BSL432_ReceiveBuffer[5]) << 8;
    addr |= ((uint32_t)BSL432_ReceiveBuffer[6]) << 16;
    /* Fourth address byte for 32-bit commands */
    if(command == BSL432_RX_DATA_BLOCK_32 ||
       command == BSL432_RX_CRC_CHECK_32 ||
       command == BSL432_RX_LOAD_PC_32 ||
       command == BSL432_TX_DATA_BLOCK_32 ||
       command == BSL432_ERASE_SECTOR_32)
    {
        addr |= ((uint32_t)BSL432_ReceiveBuffer[7]) << 24;
    }

    switch (command)
    {
        case BSL432_RX_DATA_BLOCK:
        {
            BSL432_CI_receiveDataBlock(addr, &BSL432_ReceiveBuffer[7]);
            break;
        }
        case BSL432_RX_DATA_BLOCK_32:
        {
            BSL432_CI_receiveDataBlock32(addr, &BSL432_ReceiveBuffer[8]);
            break;
        }
        case BSL432_RX_PASSWORD:
        {
            BSL432_CI_receivePassword();
            break;
        }
        case BSL432_ERASE_SECTOR:
        {
            BSL432_CI_sendMessage(BSL432_API_eraseSector(addr));
            break;
        }
        case BSL432_ERASE_SECTOR_32:
        {
            BSL432_CI_sendMessage(BSL432_API_eraseSector(addr));
            break;
        }
        case BSL432_RX_MASS_ERASE:
        {
            BSL432_CI_sendMessage(BSL432_API_massErase(BSL432_STATUS_LOCKED));
            break;
        }
        case BSL432_RX_CRC_CHECK:
        {
            uint16_t length;
            length = BSL432_ReceiveBuffer[7];
            length |= ((uint16_t)BSL432_ReceiveBuffer[8]) << 8;
            BSL432_CI_CRCCheck(addr, length);
            break;
        }
        case BSL432_RX_CRC_CHECK_32:
        {
            uint16_t length;
            length = BSL432_ReceiveBuffer[8];
            length |= ((uint16_t)BSL432_ReceiveBuffer[9]) << 8;
            BSL432_CI_CRCCheck(addr, length);
            break;
        }
        case BSL432_RX_LOAD_PC:
        {
            BSL432_CI_sendMessage(BSL432_API_callAddress(addr));
            break;
        }
        case BSL432_RX_LOAD_PC_32:
        {
            BSL432_CI_sendMessage(BSL432_API_callAddress(addr));
            break;
        }
        case BSL432_TX_DATA_BLOCK:
        {
            uint16_t length;
            length = BSL432_ReceiveBuffer[7];
            length |= ((uint16_t)BSL432_ReceiveBuffer[8]) << 8;
            BSL432_CI_sendDataBlock(addr, length);
            break;
        }
        case BSL432_TX_DATA_BLOCK_32:
        {
            uint16_t length;
            length = BSL432_ReceiveBuffer[8];
            length |= ((uint16_t)BSL432_ReceiveBuffer[9]) << 8;
            BSL432_CI_sendDataBlock(addr, length);
            break;
        }
        case BSL432_TX_BSL_VERSION:
        {
            BSL432_CI_sendDataBlock((uint32_t)(&BSL432_VersionVendor), 10);
            break;
        }
        case BSL432_REBOOT_RESET:
        {
            BSL432_API_rebootReset();
            break;
        }
        case BSL432_FACTORY_RESET:
        {
            BSL432_API_factoryReset(&BSL432_ReceiveBuffer[4]);
            break;
        }
        default:
        {
            BSL432_CI_sendMessage(BSL432_UNKNOWN_COMMAND);
            break;
        }
    }

}

/**
 * Calls the API to write a data block to a given memory location
 */
void BSL432_CI_receiveDataBlock(uint32_t addr, uint8_t *data)
{
    uint8_t returnValue = BSL432_API_openMemory();

    // Check if there is data in buffer (cmd + addr take 4 bytes, 24 bit addr)
    if((returnValue == BSL432_SUCCESSFUL_OPERATION) & (BSL432_ReceiveBufferSize > 4))
    {
        returnValue = BSL432_API_writeMemory(addr, BSL432_ReceiveBufferSize - 4, data);
    }

    BSL432_CI_sendMessage(returnValue);
    BSL432_API_closeMemory();
}

/**
 * Calls the API to write a data block to a given memory location
 */
void BSL432_CI_receiveDataBlock32(uint32_t addr, uint8_t *data)
{
    uint8_t returnValue = BSL432_API_openMemory();

    // Check if there is data in buffer (cmd + addr take 5 bytes, 32 bit addr)
    if((returnValue == BSL432_SUCCESSFUL_OPERATION) & (BSL432_ReceiveBufferSize > 5))
    {
        returnValue = BSL432_API_writeMemory(addr, BSL432_ReceiveBufferSize - 5, data);
    }

    BSL432_CI_sendMessage(returnValue);
    BSL432_API_closeMemory();
}

/**
 * Calls the API to unlock the BSL
 */
void BSL432_CI_receivePassword()
{
    if (BSL432_API_unlockBSL(&BSL432_ReceiveBuffer[4]) == BSL432_SUCCESSFUL_OPERATION)
    {
        BSL432_CI_sendMessage(BSL432_ACK);
    }
    else
    {
        BSL432_CI_sendMessage(BSL432_PASSWORD_ERROR);
    }
}

/**
 * Calls the API to perform a CRC check over a given addr and length of data
 */
void BSL432_CI_CRCCheck(uint32_t addr, uint32_t length)
{
    uint16_t crcResult = 0;
    uint32_t crcStatus = BSL432_API_CRCCheck(addr, length, &crcResult);

    if (crcStatus == BSL432_LOCKED)
    {
        BSL432_CI_sendMessage(BSL432_LOCKED);
    }
    else
    {
        BSL432_SendBuffer[3] = BSL432_DATA_REPLY;
        BSL432_SendBuffer[4] = (uint8_t)(crcResult & 0xFF);
        BSL432_SendBuffer[5] = (uint8_t)((crcResult >> 8) & 0xFF);
        BSL432_PI_sendData(3);
    }
}

/**
 * Fills the SendBuffer array with bytes from the given parameters.
 * Sends the data by calling the PI, or sends an error.
 */
void BSL432_CI_sendDataBlock(uint32_t addr, uint32_t length)
{
    uint32_t endAddr = addr + length;
    uint32_t bytes = 0;
    uint8_t returnValue = BSL432_SUCCESSFUL_OPERATION;

    while((addr < endAddr) & (returnValue == BSL432_SUCCESSFUL_OPERATION))
    {
        if((endAddr - addr) > BSL432_PI_getBufferSize() - 1)
        {
            bytes = BSL432_PI_getBufferSize() - 1;
        }
        else
        {
            bytes = (endAddr - addr);
        }
        returnValue = BSL432_API_readMemory(addr, bytes, &BSL432_SendBuffer[4]);
        if (returnValue == BSL432_SUCCESSFUL_OPERATION)
        {
            BSL432_SendBuffer[3] = BSL432_DATA_REPLY;
            BSL432_PI_sendData(bytes + 1);
        }
        else
        {
            BSL432_CI_sendMessage(returnValue);
        }
        addr += bytes;
    }
}

/**
 * Sends a reply message with attached information.
 */
void BSL432_CI_sendMessage(uint8_t message)
{
    BSL432_SendBuffer[3] = BSL432_MESSAGE_REPLY;
    BSL432_SendBuffer[4] = message;
    BSL432_PI_sendData(2);
}
